﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Petzold.Epub.Common
{
    public struct BookLocation
    {
        public BookLocation(int fileIndex, int breakIndex, int pageIndex, int charOffset)
            : this()
        {
            this.FileIndex = fileIndex;
            this.BreakIndex = breakIndex;
            this.PageIndex = pageIndex;
            this.CharacterOffset = charOffset;
        }

        public int FileIndex { set; get; }

        public int BreakIndex { set; get; }

        public int PageIndex { set; get; }

        public int CharacterOffset { set; get; }

        public bool IsFirstPage
        {
            get 
            { 
                return this.FileIndex == 0 && this.BreakIndex == 0 && this.PageIndex == 0; 
            }
        }

        public BookLocation NextFile()
        {
            return new BookLocation(this.FileIndex + 1, 0, 0, -1);
        }

        public BookLocation NextBreak()
        {
            return new BookLocation(this.FileIndex, this.BreakIndex + 1, 0, -1);
        }

        public BookLocation NextPage()
        {
            return new BookLocation(this.FileIndex, this.BreakIndex, this.PageIndex + 1, -1);
        }

        public BookLocation PreviousPage()
        {
            if (this.PageIndex > 0)
                return new BookLocation(this.FileIndex, this.BreakIndex, this.PageIndex - 1, -1);

                // TODO: Restore when breaks are implemented

  //          else if (this.BreakIndex > 0)
    //            return new BookLocation(this.FileIndex, this.BreakIndex - 1, -1, -1);

            else if (this.FileIndex > 0)
                return new BookLocation(this.FileIndex - 1, -1, -1, -1);

            return new BookLocation(-1, -1, -1, -1);
        }

        public override string ToString()
        {
            return String.Format("({0},{1},{2},{3})", this.FileIndex, this.BreakIndex, this.PageIndex, this.CharacterOffset);
        }
    }
}
