﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Petzold.Epub.Css;

namespace Petzold.Epub
{
    public class HtmlElement
    {
        public HtmlElement(HtmlElementType elementType, HtmlElement parent, int accumulatedCharacterCount)
        {
            this.ElementType = elementType;
            this.Parent = parent;
            this.AccumulatedCharacterCount = accumulatedCharacterCount;
        }

        public HtmlElement(HtmlElement parent, string text, int accumulatedCharacterCount)
        {
            this.ElementType = HtmlElementType._text;
            this.Parent = parent;
            this.Text = text;
            this.AccumulatedCharacterCount = accumulatedCharacterCount;
        }

        public HtmlElement(string elementName, HtmlElement parent, int accumulatedCharacterCount)
        {
            this.ElementName = elementName;
            this.Parent = parent;
            this.AccumulatedCharacterCount = accumulatedCharacterCount;
        }

        public HtmlElementType ElementType { protected set; get; }

        public string ElementName { protected set; get; }

        public HtmlElement Parent { protected set; get; }

        public int AccumulatedCharacterCount { protected set; get; }

        public string ID { protected set; get; }

        // xml:space, xml:lang, dir ???

        public string Class { protected set; get; }

        public string Title { protected set; get; }

        public string Style { protected set; get; }

        // Valid only for ElementType of _text.
        public string Text { protected set; get; }


        public IDictionary<string, string> Attributes { protected set; get; }

        public IList<HtmlElement> Children { protected set; get; }

        public HtmlElement NextSibling
        {
            get
            {
                if (this.Parent == null || this.Parent.Children == null)
                    return null;

                int index = this.Parent.Children.IndexOf(this);

                if (index == this.Parent.Children.Count - 1)
                    return null;

                return this.Parent.Children[index + 1];
            }
        }

        public HtmlElement PreviousSibling
        {
            get
            {
                if (this.Parent == null || this.Parent.Children == null)
                    return null;

                int index = this.Parent.Children.IndexOf(this);

                if (index == 0)
                    return null;

                return this.Parent.Children[index - 1];
            }
        }



        public void AddAttribute(string attr, string value)
        {
            switch (attr)
            {
                // "Core" attributes
                case "xml:space":
     //               System.Diagnostics.Debug.WriteLine("HtmlElement not handling xml:space attribute");
                    goto default;

                case "class":
                    this.Class = value;
                    break;

                case "id":
                    this.ID = value;
                    break;

                case "title":
                    this.Title = value;
                    break;

                // "I18N" attributes
                case "dir":
        //            System.Diagnostics.Debug.WriteLine("HtmlElement not handling dir attribute");
                    goto default;

                case "xml:lang":
      //              System.Diagnostics.Debug.WriteLine("HtmlElement not handling xml:lang attribute");
                    goto default;

                // "Style" attribute
                case "style":
                    this.Style = value;
                    break;

                // Other
                default:
        //            System.Diagnostics.Debug.WriteLine("HtmlElement not handling attribute {0} in element {1}", attr, this.ElementType);
                    if (this.Attributes == null)
                        this.Attributes = new Dictionary<string, string>();

                    this.Attributes.Add(attr, value);
                    break;
            }
        }

        public void AddChild(HtmlElement child)
        {
            if (this.Children == null)
                this.Children = new List<HtmlElement>();

            this.Children.Add(child);
        }

        public override string ToString()
        {
            if (this.ElementType == HtmlElementType._nothing || this.ElementType == HtmlElementType._unsupported)
            {
                if (String.IsNullOrEmpty(ID))
                    return this.ElementName;
                else
                    return String.Format("{0} {1}", this.ElementName, this.ID);
            }
            else if (String.IsNullOrEmpty(ID))
                return this.ElementType.ToString();

            return String.Format("{0} {1}", this.ElementType, this.ID);
        }
    }
}
