﻿using System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;

namespace Petzold.Epub.Viewer.Windows8
{
    // This is a temporary class to be replaced by a code-based Storyboard and
    //      DoubleAnimation when these can be used to target a custom property.
public class DoubleAnimator
{
    TimeSpan startTime = TimeSpan.Zero;
    double startValue;

    // These properties much be set before calling Begin!
    public DependencyObject Target { set; get; }
    public DependencyProperty Property { set; get; }
    public double ToValue { set; get; }
    public Duration Duration { set; get; }

    // Optional 
    public Windows.UI.Xaml.EventHandler Completed { set; get; }

    public void Begin()
    {
        CompositionTarget.Rendering += OnCompositionTargetRendering;
    }

    void OnCompositionTargetRendering(object sender, object args)
    {
        TimeSpan renderingTime = (args as RenderingEventArgs).RenderingTime;

        if (startTime == TimeSpan.Zero)
        {
            startTime = renderingTime;
            startValue = (double)Target.GetValue(Property);
        }
        else
        {
            double value = 0;

            if (renderingTime - startTime > Duration.TimeSpan)
            {
                Target.SetValue(Property, ToValue);
                CompositionTarget.Rendering -= OnCompositionTargetRendering;
                if (Completed != null)
                    Completed(this, EventArgs.Empty);
            }
            else
            {
                double fraction = (double)(renderingTime - startTime).Ticks / 
                                                Duration.TimeSpan.Ticks;
                value = startValue * (1 - fraction) + ToValue * fraction;
                Target.SetValue(Property, value);
            }
        }
    }
}
}
