﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using Petzold.Epub.Common;
using Petzold.Epub.Html;

namespace Petzold.Epub
{
    public partial class EpubBook
    {
        IPackageAccessor packageAccessor;
        IFontMetricsProvider fontMetricsProvider;
        BookInformation bookInformation;
        TableOfContents tableOfContents;
        IList<HtmlFile> htmlFiles;
        string navigationFilePath;

  //      HtmlFile[] htmlFiles;

        public EpubBook(string packageFilePath, IPackageAccessor packageAccessor, IFontMetricsProvider fontMetricsProvider)
        {
            this.PackageFilePath = packageFilePath;
            this.packageAccessor = packageAccessor;
            this.fontMetricsProvider = fontMetricsProvider;

            // Find directory preface for packageAccessor
            int index = packageFilePath.LastIndexOf('/');

            if (index != -1)
            {
                packageAccessor.DirectoryPreface = packageFilePath.Substring(0, index + 1);
            }
        }

        public string PackageFilePath { protected set; get; }

        public BookInformation BookInformation 
        {
            get
            {
                if (bookInformation == null)
                {
                    try
                    {
                        ParseOpfFile();
                    }
                    catch (Exception exc)
                    {
                        System.Diagnostics.Debug.WriteLine("From BookInformation: " + exc);
                    }
                }
                return bookInformation;
            }
        }

        public TableOfContents TableOfContents 
        {
            get
            {
                if (tableOfContents == null)
                {
                    if (String.IsNullOrEmpty(navigationFilePath))
                    {
                        try
                        {
                            ParseOpfFile();
                        }
                        catch (Exception exc)
                        {
                            System.Diagnostics.Debug.WriteLine("From TableOfContents: " + exc);
                        }
                    }

                    try
                    {
                        ParseNcxFile();
                    }
                    catch (Exception exc)
                    {
                        System.Diagnostics.Debug.WriteLine("From TableOfContents: " + exc);
                    }
                }
                return tableOfContents;
            }
        }

        public IList<HtmlFile> HtmlFiles
        {
            get
            {
                if (htmlFiles == null)
                    ParseOpfFile();
                    
                return htmlFiles;
            }
        }

        public Stream OpenFile(string filename)
        {
            return packageAccessor.GetFile(filename);
        }

        public PageProvider GetPageProvider()
        {
            return new PageProvider(this.HtmlFiles);
        }
    }
}
