﻿using System.IO;
using System.Xml.Serialization;

// Re "Open Packaging Format (OPF) 2.0.1 v1.0.1
namespace Petzold.Epub.Package.OpfPackageDocument
{
    [XmlRoot(Namespace="http://www.idpf.org/2007/opf")]
    public class package
    {
        public static package Deserialize(Stream stream)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(package));
            return serializer.Deserialize(stream) as package;
        }

        [XmlNamespaceDeclarations]
        public XmlSerializerNamespaces xmlns { set; get; }

        [XmlAttribute]
        public string version;

        [XmlAttribute(AttributeName = "unique-identifier")]
        public string uniqueIdentifier;

        public metadata metadata { set; get; }

        public manifest manifest { set; get; }

        public spine spine { set; get; }

        public guide guide { set; get; }
    }

    // This is not complete!
    // From "Open Package Format (OPF), section 2.2
    public class metadata
    {
        [XmlElement(Namespace = "http://purl.org/dc/elements/1.1/")]
        public string[] title { set; get; }

        [XmlElement(ElementName = "creator", Namespace = "http://purl.org/dc/elements/1.1/")]
        public creator[] creators { set; get; }

        [XmlElement(ElementName = "subject", Namespace = "http://purl.org/dc/elements/1.1/")]
        public string[] subjects { set; get; }

        [XmlElement(Namespace = "http://purl.org/dc/elements/1.1/")]
        public string description { set; get; }

        [XmlElement(Namespace = "http://purl.org/dc/elements/1.1/")]
        public string publisher { set; get; }

        // Note: contributor is same data type as creator
        [XmlElement(ElementName = "contributor", Namespace = "http://purl.org/dc/elements/1.1/")]
        public creator[] contributors { set; get; }

        [XmlElement(ElementName = "date", Namespace = "http://purl.org/dc/elements/1.1/")]
        public date[] dates { set; get; }

        // type, format, identifier are not yet included here

        [XmlElement(Namespace = "http://purl.org/dc/elements/1.1/")]
        public string source { set; get; }

        // language, relation, coverage are not yet included here

        [XmlElement(Namespace = "http://purl.org/dc/elements/1.1/")]
        public string rights { set; get; }

        [XmlElement(ElementName = "meta")]
        public meta[] metas { set; get; }
    }

    public class creator
    {
        [XmlAttribute(Namespace = "http://www.idpf.org/2007/opf")]
        public string role;

        [XmlAttribute(AttributeName = "file-as", Namespace = "http://www.idpf.org/2007/opf")]
        public string fileas;

        [XmlText]
        public string name;
    }

    public class date
    {
        [XmlAttribute(AttributeName = "event", Namespace = "http://www.idpf.org/2007/opf")]
        public string evt;

        [XmlText]
   //     [XmlElement(DataType="date")]
        public string value;
    }

    public class meta
    {
        [XmlAttribute]
        public string content { set; get; }

        [XmlAttribute]
        public string name { set; get; }
    }

    public class manifest
    {
        [XmlElement(ElementName="item")]
        public item[] items { set; get; }
    }

    public class item
    {
        [XmlAttribute]
        public string href;

        [XmlAttribute]
        public string id;

        [XmlAttribute(AttributeName="media-type")]
        public string mediaType;
    }

    public class spine
    {
        [XmlAttribute]
        public string toc;

        [XmlElement(ElementName = "itemref")]
        public itemref[] itemrefs;
    }

    public class itemref
    {
        [XmlAttribute]
        public string idref;

        [XmlAttribute]
        public string linear;
    }

    public class guide
    {
        public reference[] reference;
    }

    public class reference
    {
        [XmlAttribute]
        public string href;

        [XmlAttribute]
        public string type;

        [XmlAttribute]
        public string title;
    }
}
