﻿using System;
using System.IO;
using System.Threading.Tasks;
using System.Xml.Serialization;
using Windows.Storage;
using Windows.Storage.Streams;

namespace TheNewEpublic
{
    public class AppSettings
    {
        const string FILENAME = "applicationsettings.xml";

        public AppSettings()
        {
            this.Library = new Library();
        }

        public Library Library { set; get; }

        public static Task<AppSettings> LoadAsync()
        {
            return Task.Run<AppSettings>(async () =>
            {
                StorageFolder storageFolder = ApplicationData.Current.LocalFolder;
                StorageFile appSettingsFile = null;
                AppSettings appSettings = null;

                try
                {
                    appSettingsFile = await storageFolder.GetFileAsync(FILENAME);
                }
                catch (Exception)
                {
                    System.Diagnostics.Debug.WriteLine("Can't find " + FILENAME);
                }

                if (appSettingsFile == null || appSettingsFile.Size == 0)
                {
                    appSettings = new AppSettings();
                }
                else
                {
                    ulong size = appSettingsFile.Size;
                    IInputStream inputStream = await appSettingsFile.OpenForReadAsync();
                    DataReader dataReader = new DataReader(inputStream);
                    await dataReader.LoadAsync((uint)size);
                    byte[] buffer = new byte[(int)size];
                    dataReader.ReadBytes(buffer);

                    using (MemoryStream memoryStream = new MemoryStream(buffer, false))
                    {
                        XmlSerializer xmlSerializer = new XmlSerializer(typeof(AppSettings));
                        appSettings = xmlSerializer.Deserialize(memoryStream) as AppSettings;
                    }
                }

                return appSettings;
            });
        }

        public Task SaveAsync()
        {
            return Task.Run(async () => 
                {
                    byte[] buffer = null;

                    using (MemoryStream memoryStream = new MemoryStream())
                    {
                        XmlSerializer xmlSerializer = new XmlSerializer(typeof(AppSettings));
                        xmlSerializer.Serialize(memoryStream, this);
                        buffer = memoryStream.ToArray();
                    }
                    StorageFolder storageFolder = ApplicationData.Current.LocalFolder;
                    StorageFile libraryFile = await storageFolder.CreateFileAsync(FILENAME, CreationCollisionOption.ReplaceExisting);
                    IRandomAccessStream randomAccessStream = await libraryFile.OpenAsync(FileAccessMode.ReadWrite);
                    IOutputStream outputStream = randomAccessStream.GetOutputStreamAt(0);
                    DataWriter dataWriter = new DataWriter(outputStream);
                    dataWriter.WriteBytes(buffer);

                    await dataWriter.StoreAsync();
                    await outputStream.FlushAsync();
                });
        }
    }
}
