﻿using System;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Petzold.Epub;
using Petzold.Epub.Common;

namespace TheNewEpublic
{
    public class FontMetricsProvider : IFontMetricsProvider
    {
        public TextSize GetSize(IWord word)
        {
            TextBlock txtblk = CreateTextBlock(word);
            TextBlock spaceTxtBlk = null;

            if (word.Text == " ")
                spaceTxtBlk = txtblk;
            else
                spaceTxtBlk = CreateTextBlock(word.CloneWithNewText(" "));

            return new TextSize(txtblk.ActualWidth, txtblk.ActualHeight, 0, spaceTxtBlk.ActualWidth);
        }

        public TextSize GetIntegerSize(IWord word)
        {
            TextBlock txtblk = CreateTextBlock(word);
            TextBlock spaceTxtBlk = null;

            if (word.Text == " ")
                spaceTxtBlk = txtblk;
            else
                spaceTxtBlk = CreateTextBlock(word.CloneWithNewText(" "));

            txtblk.Measure(new Size(Double.PositiveInfinity, Double.PositiveInfinity));
            spaceTxtBlk.Measure(new Size(Double.PositiveInfinity, Double.PositiveInfinity));

            return new TextSize(txtblk.DesiredSize.Width, txtblk.DesiredSize.Height, 0, spaceTxtBlk.DesiredSize.Width);
        }

        TextBlock CreateTextBlock(IWord word)
        {
            TextBlock txtblk = new TextBlock
            {
                FontStyle = word.IsItalic ? FontStyles.Italic : FontStyles.Normal,
                FontWeight = word.IsBold ? FontWeights.Bold : FontWeights.Normal,
                Text = word.Text
            };

            if (word.FontSize > 0)
                txtblk.FontSize = word.FontSize;

            if (!String.IsNullOrEmpty(word.FontFamily))
                txtblk.FontFamily = new FontFamily { Source = word.FontFamily };

            return txtblk;
        }
    }
}
