﻿using System;
using System.Collections.Generic;
using System.IO;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Petzold.Epub;

namespace TheNewEpublic
{
    partial class MainPage
    {
        AppSettings appSettings;

        public MainPage()
        {
            InitializeComponent();
            Loaded += OnLoaded;
        }

        async void OnLoaded(object sender, RoutedEventArgs args)
        {
            // Get application settings for library view
            appSettings = await AppSettings.LoadAsync();
            libraryGridView.ItemsSource = appSettings.Library.Books;
            appbar.IsOpen = appSettings.Library.Books.Count == 0;

            (Application.Current as App).AppSettings = appSettings;
        }

        async void OnOpenFileClick(object sender, RoutedEventArgs args)
        {
            appbar.IsOpen = false;

            FileOpenPicker picker = new FileOpenPicker();
            picker.FileTypeFilter.Add(".epub");
            picker.CommitButtonText = "Open";
            picker.ViewMode = PickerViewMode.List;

            StorageFile storageFile = await picker.PickSingleFileAsync();

            if (storageFile == null)
                return;

            byte[] buffer = await Helpers.ReadStorageFileAsync(storageFile);

            using (MemoryStream memoryStream = new MemoryStream(buffer, false))
            {
                ZipPackageAccessor packageAccessor = new ZipPackageAccessor(memoryStream);
                IList<EpubBook> books = null;
                Exception exception = null;

                try
                {
                    books = EpubBook.GetAllBooksInPackage(packageAccessor);
                }
                catch (Exception exc)
                {
                    exception = exc;
                }

                if (exception != null)
                {
                    MessageDialog messageDialog = new MessageDialog("The EPUB file does not include a required mimetype entry", "epub file error");
                    await messageDialog.ShowAsync();
                    return;
                }

                if (books.Count == 0)
                {
                    MessageDialog messageDialog = new MessageDialog("The EPUB file does not contain a book in the EPUB format", "epub file error");
                    await messageDialog.ShowAsync();
                    return;
                }

                System.Diagnostics.Debug.WriteLine("Enumerating through books");

                foreach (EpubBook book in books)
                {
                    BookInformation information = null;

                    try
                    {
                        information = book.BookInformation;
                    }
                    catch (Exception exc)
                    {
                        System.Diagnostics.Debug.WriteLine("Bad information: " + exc);
                        return;
                    }

                    BookReference bookReference = new BookReference(storageFile.Path, book.PackageFilePath, information);
                    appSettings.Library.Books.Add(bookReference);

                    // Why is this necessary to get the GridView to recognize added items?
                    libraryGridView.ItemsSource = null;
                    libraryGridView.ItemsSource = appSettings.Library.Books;
                }
            }

            await appSettings.SaveAsync();
            return;
/*
            using (MemoryStream memoryStream = new MemoryStream(buffer, false))
            {
                ZipArchive zipArchive = new ZipArchive(memoryStream, ZipArchiveMode.Read);
                ReadOnlyCollection<ZipArchiveEntry> entries = zipArchive.Entries;

                foreach (ZipArchiveEntry entry in entries)
                {
                    System.Diagnostics.Debug.WriteLine(entry.FullName);

                                    if (entry.FullName.EndsWith(".xml"))
                                    {
                                        System.Diagnostics.Debug.WriteLine("Creating StreamReader for " + entry.FullName);

                                        StreamReader reader = new StreamReader(entry.Open());
                                        string container = reader.ReadToEnd();

                                        System.Diagnostics.Debug.WriteLine("file length = " + container.Length);
                                        System.Diagnostics.Debug.WriteLine(container);

                                        //           txtblk.Dispatcher.Invoke(Windows.UI.Core.CoreDispatcherPriority.Normal, (sender, args) =>
                                        //             {

                                        txtblk.Text = container;
                                        //           }, this, null);
                                    }

                                    if (entry.FullName.EndsWith(".png"))
                                    {
                                        MemoryStream memStream = new MemoryStream();
                                        Stream entryStream = entry.Open();
                                        entryStream.CopyTo(memStream);
                                        memStream.Seek(0, SeekOrigin.Begin);

                                        System.Diagnostics.Debug.WriteLine("memstream Length = " + memStream.Length);

                                        byte[] memBuffer = new byte[memStream.Length];
                                        memBuffer = memStream.ToArray();



                                        InMemoryRandomAccessStream randomStream = new InMemoryRandomAccessStream();
                                        IOutputStream outputStream = randomStream.GetOutputStreamAt(0);
                                        DataWriter writer = new DataWriter(outputStream);
                                        writer.WriteBytes(memBuffer);
                                        await writer.StoreAsync();

                                        System.Diagnostics.Debug.WriteLine(randomStream.Size);





                                        BitmapImage bitmap = new BitmapImage();
                                        bitmap.SetSource(randomStream);
                                        image.Source = bitmap;
                                    }
  

                }
            }
*/
        }

        void OnLibraryGridViewItemClick(object sender, ItemClickEventArgs args)
        {
            Window.Current.Content = new BookViewerPage(args.ClickedItem as BookReference);
        }

        void OnLibraryGridViewSelectionChanged(object sender, SelectionChangedEventArgs args)
        {


        }

        private void OnBackButtonClick(object sender, RoutedEventArgs args)
        {
            // TODO: Confirm that this is the right way to exit an app
            Application.Current.Exit();
        }
    }
}
